from enum import Enum


class GetFeaturesResponseFeaturesItem(str, Enum):
    TELEMETRY = "telemetry"
    INTERNALTELEMETRY = "internalTelemetry"
    TELEOP = "teleop"
    SSH = "ssh"
    CUSTOMEVENTS = "customEvents"
    TRIGGEREDEVENTS = "triggeredEvents"
    PORTFORWARDING = "portForwarding"
    COMMANDS = "commands"
    INTERVENTIONS = "interventions"
    ONDEMAND = "onDemand"
    APPCONFIG = "appConfig"
    BLOBSTORAGE = "blobStorage"
    EVENTS = "events"
    ANALYTICS = "analytics"
    ANNOTATIONS = "annotations"
    OBSERVABILITY = "observability"
    DIAGNOSTICS = "diagnostics"
    ADVANCEDCONFIGURATION = "advancedConfiguration"
    DATAEXPORT = "dataExport"
    SHARE = "share"
    ADAPTERS = "adapters"
    S3EXPORT = "s3Export"
    FILESTORAGE = "fileStorage"
    ROLEVIEWER = "roleViewer"
    TEAMS = "teams"
    SCHEDULES = "schedules"
    TELEOP2_0 = "teleop2.0"
    BILLING = "billing"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Dict, Optional

import httpx

from ...client import AuthenticatedClient
from ...models.user_list_response import UserListResponse
from ...types import Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
) -> Dict[str, Any]:
    url = "{}/users".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[UserListResponse]:
    if response.status_code == HTTPStatus.OK:
        response_200 = UserListResponse.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[UserListResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
) -> Response[UserListResponse]:
    """Get all

     List all users
    Resource: users
    Authorized roles: viewer

    Returns:
        Response[UserListResponse]
    """

    kwargs = _get_kwargs(
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
) -> Optional[UserListResponse]:
    """Get all

     List all users
    Resource: users
    Authorized roles: viewer

    Returns:
        Response[UserListResponse]
    """

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
) -> Response[UserListResponse]:
    """Get all

     List all users
    Resource: users
    Authorized roles: viewer

    Returns:
        Response[UserListResponse]
    """

    kwargs = _get_kwargs(
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
) -> Optional[UserListResponse]:
    """Get all

     List all users
    Resource: users
    Authorized roles: viewer

    Returns:
        Response[UserListResponse]
    """

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

if TYPE_CHECKING:
    from ..models.annotation import Annotation
    from ..models.command_delivery_event import CommandDeliveryEvent
    from ..models.comment import Comment
    from ..models.custom_event import CustomEvent
    from ..models.datapoint_event import DatapointEvent
    from ..models.device_offline_event import DeviceOfflineEvent
    from ..models.device_online_event import DeviceOnlineEvent
    from ..models.intervention_request import InterventionRequest
    from ..models.port_forwarding_session_record import PortForwardingSessionRecord
    from ..models.system_event import SystemEvent
    from ..models.teleop_session_record import TeleopSessionRecord


T = TypeVar("T", bound="EventListResponse")


@attr.s(auto_attribs=True)
class EventListResponse:
    """
    Attributes:
        items (List[Union['Annotation', 'CommandDeliveryEvent', 'Comment', 'CustomEvent', 'DatapointEvent',
            'DeviceOfflineEvent', 'DeviceOnlineEvent', 'InterventionRequest', 'PortForwardingSessionRecord', 'SystemEvent',
            'TeleopSessionRecord']]):
    """

    items: List[
        Union[
            "Annotation",
            "CommandDeliveryEvent",
            "Comment",
            "CustomEvent",
            "DatapointEvent",
            "DeviceOfflineEvent",
            "DeviceOnlineEvent",
            "InterventionRequest",
            "PortForwardingSessionRecord",
            "SystemEvent",
            "TeleopSessionRecord",
        ]
    ]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.command_delivery_event import CommandDeliveryEvent
        from ..models.comment import Comment
        from ..models.custom_event import CustomEvent
        from ..models.datapoint_event import DatapointEvent
        from ..models.device_offline_event import DeviceOfflineEvent
        from ..models.device_online_event import DeviceOnlineEvent
        from ..models.intervention_request import InterventionRequest
        from ..models.port_forwarding_session_record import PortForwardingSessionRecord
        from ..models.system_event import SystemEvent
        from ..models.teleop_session_record import TeleopSessionRecord

        items = []
        for items_item_data in self.items:
            items_item: Dict[str, Any]

            if isinstance(items_item_data, DatapointEvent):
                items_item = items_item_data.to_dict()

            elif isinstance(items_item_data, DeviceOnlineEvent):
                items_item = items_item_data.to_dict()

            elif isinstance(items_item_data, DeviceOfflineEvent):
                items_item = items_item_data.to_dict()

            elif isinstance(items_item_data, InterventionRequest):
                items_item = items_item_data.to_dict()

            elif isinstance(items_item_data, TeleopSessionRecord):
                items_item = items_item_data.to_dict()

            elif isinstance(items_item_data, PortForwardingSessionRecord):
                items_item = items_item_data.to_dict()

            elif isinstance(items_item_data, CommandDeliveryEvent):
                items_item = items_item_data.to_dict()

            elif isinstance(items_item_data, CustomEvent):
                items_item = items_item_data.to_dict()

            elif isinstance(items_item_data, Comment):
                items_item = items_item_data.to_dict()

            elif isinstance(items_item_data, SystemEvent):
                items_item = items_item_data.to_dict()

            else:
                items_item = items_item_data.to_dict()

            items.append(items_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "items": items,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.annotation import Annotation
        from ..models.command_delivery_event import CommandDeliveryEvent
        from ..models.comment import Comment
        from ..models.custom_event import CustomEvent
        from ..models.datapoint_event import DatapointEvent
        from ..models.device_offline_event import DeviceOfflineEvent
        from ..models.device_online_event import DeviceOnlineEvent
        from ..models.intervention_request import InterventionRequest
        from ..models.port_forwarding_session_record import PortForwardingSessionRecord
        from ..models.system_event import SystemEvent
        from ..models.teleop_session_record import TeleopSessionRecord

        d = src_dict.copy()
        items = []
        _items = d.pop("items")
        for items_item_data in _items:

            def _parse_items_item(
                data: object,
            ) -> Union[
                "Annotation",
                "CommandDeliveryEvent",
                "Comment",
                "CustomEvent",
                "DatapointEvent",
                "DeviceOfflineEvent",
                "DeviceOnlineEvent",
                "InterventionRequest",
                "PortForwardingSessionRecord",
                "SystemEvent",
                "TeleopSessionRecord",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    items_item_type_0 = DatapointEvent.from_dict(data)

                    return items_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    items_item_type_1 = DeviceOnlineEvent.from_dict(data)

                    return items_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    items_item_type_2 = DeviceOfflineEvent.from_dict(data)

                    return items_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    items_item_type_3 = InterventionRequest.from_dict(data)

                    return items_item_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    items_item_type_4 = TeleopSessionRecord.from_dict(data)

                    return items_item_type_4
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    items_item_type_5 = PortForwardingSessionRecord.from_dict(data)

                    return items_item_type_5
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    items_item_type_6 = CommandDeliveryEvent.from_dict(data)

                    return items_item_type_6
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    items_item_type_7 = CustomEvent.from_dict(data)

                    return items_item_type_7
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    items_item_type_8 = Comment.from_dict(data)

                    return items_item_type_8
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    items_item_type_9 = SystemEvent.from_dict(data)

                    return items_item_type_9
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                items_item_type_10 = Annotation.from_dict(data)

                return items_item_type_10

            items_item = _parse_items_item(items_item_data)

            items.append(items_item)

        event_list_response = cls(
            items=items,
        )

        event_list_response.additional_properties = d
        return event_list_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ViewLayoutType(str, Enum):
    OBSERVE = "observe"
    FULLSCREEN = "fullscreen"
    ANALYTICS = "analytics"
    TELEOP = "teleop"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.network import Network


T = TypeVar("T", bound="NetworkInfo")


@attr.s(auto_attribs=True)
class NetworkInfo:
    """
    Attributes:
        networks (List['Network']):
    """

    networks: List["Network"]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        networks = []
        for networks_item_data in self.networks:
            networks_item = networks_item_data.to_dict()

            networks.append(networks_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "networks": networks,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.network import Network

        d = src_dict.copy()
        networks = []
        _networks = d.pop("networks")
        for networks_item_data in _networks:
            networks_item = Network.from_dict(networks_item_data)

            networks.append(networks_item)

        network_info = cls(
            networks=networks,
        )

        network_info.additional_properties = d
        return network_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from . import Response, DeviceListResponse, DeviceQuery, PartialDevice, Device, Optional, DeviceConfiguration, device_controller_query, device_controller_patch, device_controller_get_one, device_controller_get_configuration, device_controller_post_configuration
from formant.sdk.cloud.v2.src.resources.resources import Resources

class Devices(Resources):

    def query(self, device_query: DeviceQuery):
        'Query devices by name and/or tags'
        client = self._get_client()
        response: Response[DeviceListResponse] = device_controller_query.sync_detailed(client=client, json_body=device_query)
        return response

    async def query_async(self, device_query: DeviceQuery):
        'Query devices by name and/or tags'
        client = self._get_client()
        response: Response[DeviceListResponse] = (await device_controller_query.asyncio_detailed(client=client, json_body=device_query))
        return response

    def patch(self, device_id: str, partial_device: PartialDevice):
        'Update a device'
        client = self._get_client()
        response: Response[Device] = device_controller_patch.sync_detailed(client=client, id=device_id, json_body=partial_device)
        return response

    async def patch_async(self, device_id: str, partial_device: PartialDevice):
        'Update a device'
        client = self._get_client()
        response: Response[Device] = (await device_controller_patch.asyncio_detailed(client=client, id=device_id, json_body=partial_device))
        return response

    def get_device(self, device_id: str):
        'Get a device'
        client = self._get_client()
        response: Response[Device] = device_controller_get_one.sync_detailed(client=client, id=device_id)
        return response

    async def get_device_async(self, device_id: str):
        'Get a device'
        client = self._get_client()
        response: Response[Device] = (await device_controller_get_one.asyncio_detailed(client=client, id=device_id))
        return response

    def get_device_configuration(self, device_id: str, desired_configuration_version: int):
        'Get a device configuration'
        client = self._get_client()
        response: Response[Optional[DeviceConfiguration]] = device_controller_get_configuration.sync_detailed(client=client, id=device_id, version=desired_configuration_version)
        return response

    async def get_device_configuration_async(self, device_id: str, desired_configuration_version: int):
        'Get a device configuration'
        client = self._get_client()
        response: Response[Optional[DeviceConfiguration]] = (await device_controller_get_configuration.asyncio_detailed(client=client, id=device_id, version=desired_configuration_version))
        return response

    def post_device_configuration(self, device_id, device_configuration: DeviceConfiguration):
        'Create a device configuration'
        client = self._get_client()
        response: Response[Optional[DeviceConfiguration]] = device_controller_post_configuration.sync_detailed(client=client, id=device_id, json_body=device_configuration)
        return response

    async def post_device_configuration_async(self, device_id, device_configuration: DeviceConfiguration):
        'Create a device configuration'
        client = self._get_client()
        response: Response[Optional[DeviceConfiguration]] = (await device_controller_post_configuration.asyncio_detailed(client=client, id=device_id, json_body=device_configuration))
        return response




from formant.sdk.cloud.v2.formant_admin_api_client.models import EventFilterTypesItem, DatapointEventSeverity, DeviceOfflineEventStreamType, AnnotationTags, DatapointEvent, EventHistogramEntry, DeviceOnlineEventSeverity, InterventionRequestType, EventCounts, InterventionRequestInterventionType, InterventionRequestSeverity, TeleopSessionRecordStreamType, CustomEventTags, EventQueryTypesItem, Annotation, InterventionRequest, DeviceOfflineEvent, EventExportSheetResult, EventSeekQueryTypesItem, CommandDeliveryEvent, TeleopRequestData, CommentType, EventFilterSeveritiesItem, EventSeekQueryDirection, EventSeekQuerySeveritiesItem, CustomEventSeverity, SystemEventTags, PortForwardingSessionRecord, CommandDeliveryEventType, EventSortOrder, EventHistogram, EventQuery, DatapointEventTags, EventFilterEventTypesItem, SystemEventType, TeleopSessionRecordType, TeleopSessionRecord, DeviceOfflineEventType, Label, EventSortColumn, DeviceOnlineEvent, EventSeekQuery, CustomEventStreamType, Comment, LabelingRequestData, EventQueryEventTypesItem, CommandDeliveryEventTags, CustomEventType, EventSort, DeviceOnlineEventStreamType, InterventionRequestStreamType, DeviceOnlineEventTags, SelectionRequestData, TeleopSessionRecordTags, EventQuerySeveritiesItem, PortForwardingSessionRecordTags, DeviceOfflineEventTags, InterventionRequestTags, EventExportSheetRequest, EventTriggerCommand, CommandDeliveryEventStreamType, DeviceOnlineEventType, CustomEvent, EventListResponse, DatapointEventStreamType, DatapointEventType, SystemEventStreamType, PortForwardingSessionRecordStreamType, AnnotationStreamType, EventSeekQueryEventTypesItem, DeviceOfflineEventSeverity, AnnotationType, CommentTags, EventFilter, LabeledPolygon, CommentStreamType, SystemEvent, PortForwardingSessionRecordType
from formant.sdk.cloud.v2.formant_admin_api_client.api.event import event_controller_query
from formant.sdk.cloud.v2.formant_admin_api_client.types import *



from formant.protos.model.v1 import navigation_pb2
from .transform import Transform
from .twist import Twist
from formant.sdk.agent.v1.localization.utils import validate_type, get_ros_module


class Odometry:
    @classmethod
    def from_ros(cls, odometry):
        nav_msgs = get_ros_module("nav_msgs.msg")
        validate_type(odometry, nav_msgs.Odometry)
        return cls(
            pose=Transform.from_ros_pose_with_covariance(odometry.pose),
            twist=Twist.from_ros_twist_with_covariance(odometry.twist),
        )

    @classmethod
    def from_pose_with_covariance_stamped(cls, pose_msg):
        geometry_msgs = get_ros_module("geometry_msgs.msg")
        validate_type(pose_msg, geometry_msgs.PoseWithCovarianceStamped)
        return cls(
            pose=Transform.from_ros_pose(pose_msg.pose.pose)
        )

    @classmethod
    def from_xy_direction(
        cls,
        x,  # type: float
        y,  # type: float
        direction,  # type:float
    ):
        return cls(pose=Transform.from_xy_direction(x, y, direction))

    def __init__(
        self,
        pose=Transform(),  # type: Transform
        twist=Twist(),  # type: Twist
        transform_to_world=Transform(),  # type: Transform
    ):
        self.pose = pose
        self.twist = twist
        self.transform_to_world = transform_to_world

    def to_proto(self):
        return navigation_pb2.Odometry(
            pose=self.pose.to_proto(),
            twist=self.twist.to_proto(),
            world_to_local=self.transform_to_world.to_proto(),
        )



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.partial_schedule_type import PartialScheduleType
from ..types import UNSET, Unset

T = TypeVar("T", bound="PartialSchedule")


@attr.s(auto_attribs=True)
class PartialSchedule:
    """
    Attributes:
        organization_id (Union[Unset, str]):
        name (Union[Unset, str]):
        description (Union[Unset, str]):
        type (Union[Unset, PartialScheduleType]):
        at (Union[Unset, None, datetime.datetime]):
        cron (Union[Unset, None, str]):
        duration_ms (Union[Unset, int]):
        timezone (Union[Unset, str]):
        command_template_id (Union[Unset, None, str]):
        parameter_value (Union[Unset, None, str]):
        device_id (Union[Unset, str]):
        status (Union[Unset, str]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
    """

    organization_id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    type: Union[Unset, PartialScheduleType] = UNSET
    at: Union[Unset, None, datetime.datetime] = UNSET
    cron: Union[Unset, None, str] = UNSET
    duration_ms: Union[Unset, int] = UNSET
    timezone: Union[Unset, str] = UNSET
    command_template_id: Union[Unset, None, str] = UNSET
    parameter_value: Union[Unset, None, str] = UNSET
    device_id: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        organization_id = self.organization_id
        name = self.name
        description = self.description
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        at: Union[Unset, None, str] = UNSET
        if not isinstance(self.at, Unset):
            at = self.at.isoformat() if self.at else None

        cron = self.cron
        duration_ms = self.duration_ms
        timezone = self.timezone
        command_template_id = self.command_template_id
        parameter_value = self.parameter_value
        device_id = self.device_id
        status = self.status
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if type is not UNSET:
            field_dict["type"] = type
        if at is not UNSET:
            field_dict["at"] = at
        if cron is not UNSET:
            field_dict["cron"] = cron
        if duration_ms is not UNSET:
            field_dict["durationMs"] = duration_ms
        if timezone is not UNSET:
            field_dict["timezone"] = timezone
        if command_template_id is not UNSET:
            field_dict["commandTemplateId"] = command_template_id
        if parameter_value is not UNSET:
            field_dict["parameterValue"] = parameter_value
        if device_id is not UNSET:
            field_dict["deviceId"] = device_id
        if status is not UNSET:
            field_dict["status"] = status
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        organization_id = d.pop("organizationId", UNSET)

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, PartialScheduleType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = PartialScheduleType(_type)

        _at = d.pop("at", UNSET)
        at: Union[Unset, None, datetime.datetime]
        if _at is None:
            at = None
        elif isinstance(_at, Unset):
            at = UNSET
        else:
            at = isoparse(_at)

        cron = d.pop("cron", UNSET)

        duration_ms = d.pop("durationMs", UNSET)

        timezone = d.pop("timezone", UNSET)

        command_template_id = d.pop("commandTemplateId", UNSET)

        parameter_value = d.pop("parameterValue", UNSET)

        device_id = d.pop("deviceId", UNSET)

        status = d.pop("status", UNSET)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        partial_schedule = cls(
            organization_id=organization_id,
            name=name,
            description=description,
            type=type,
            at=at,
            cron=cron,
            duration_ms=duration_ms,
            timezone=timezone,
            command_template_id=command_template_id,
            parameter_value=parameter_value,
            device_id=device_id,
            status=status,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
        )

        partial_schedule.additional_properties = d
        return partial_schedule

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.aws_info_output_format import AwsInfoOutputFormat
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.s3_export import S3Export


T = TypeVar("T", bound="AwsInfo")


@attr.s(auto_attribs=True)
class AwsInfo:
    """
    Attributes:
        account_id (str):
        role_name (Any):
        s_3_export (Union[Unset, S3Export]):
        output_format (Union[Unset, AwsInfoOutputFormat]):
    """

    account_id: str
    role_name: Any
    s_3_export: Union[Unset, "S3Export"] = UNSET
    output_format: Union[Unset, AwsInfoOutputFormat] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        account_id = self.account_id
        role_name = self.role_name
        s_3_export: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.s_3_export, Unset):
            s_3_export = self.s_3_export.to_dict()

        output_format: Union[Unset, str] = UNSET
        if not isinstance(self.output_format, Unset):
            output_format = self.output_format.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "accountId": account_id,
                "roleName": role_name,
            }
        )
        if s_3_export is not UNSET:
            field_dict["s3Export"] = s_3_export
        if output_format is not UNSET:
            field_dict["outputFormat"] = output_format

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.s3_export import S3Export

        d = src_dict.copy()
        account_id = d.pop("accountId")

        role_name = d.pop("roleName")

        _s_3_export = d.pop("s3Export", UNSET)
        s_3_export: Union[Unset, S3Export]
        if isinstance(_s_3_export, Unset):
            s_3_export = UNSET
        else:
            s_3_export = S3Export.from_dict(_s_3_export)

        _output_format = d.pop("outputFormat", UNSET)
        output_format: Union[Unset, AwsInfoOutputFormat]
        if isinstance(_output_format, Unset):
            output_format = UNSET
        else:
            output_format = AwsInfoOutputFormat(_output_format)

        aws_info = cls(
            account_id=account_id,
            role_name=role_name,
            s_3_export=s_3_export,
            output_format=output_format,
        )

        aws_info.additional_properties = d
        return aws_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Callable, Dict, Optional, Type
import json
import jsonschema
import logging
from threading import Lock
from functools import reduce

from formant.sdk.agent.v1.client import Client as AgentClient

logging.basicConfig()

JSONDecodeError = Exception  # type: Type[Exception]
try:
    JSONDecodeError = json.decoder.JSONDecodeError
except AttributeError:
    # python2 doesn't have JSON Decode Error
    pass


class JsonSchemaValidator:
    def __init__(
        self,
        client,  # type: AgentClient
        adapter_name,  # type: str
        update_adapter_config_callback,  # type: Callable[[Dict], None]
        validate=True,  # type: bool
        logger=None,  # type: Optional[logging.Logger]
        logger_level=logging.INFO,  # type: int
    ):
        self._lock = Lock()
        self._client = client
        self._adapter_name = adapter_name
        self._update_adapter_config_callback = update_adapter_config_callback
        self._validate = validate
        if logger is None:
            logger = logging.getLogger(adapter_name)
            logger.setLevel(logger_level)
        self.logger = logger
        self._client.register_config_update_callback(self._update_adapter_config)
        if self._client.ignore_unavailable:
            self._update_adapter_config()

    def _update_adapter_config(self):
        # Consumer might not be threadsafe
        with self._lock:
            try:
                configs = [
                    self._get_config_from_adapter_configuration(),
                    self._get_config_from_blob_data(),
                    self._get_config_from_json(),
                ]
                config = reduce(lambda s1, s2: s1 or s2, configs)
                if config is None:
                    raise Exception(
                        "Could not get configuration for '%s'" % self._adapter_name
                    )
                if self._validate:
                    self._validate_schema(config)
                try:
                    self._update_adapter_config_callback(config)
                except Exception as e:
                    self.logger.error(
                        "Error calling update config callback %s" % str(e)
                    )
            except Exception as e:
                self.logger.warn("Failed to load config: %s" % str(e))
                self._client.create_event(
                    "%s configuration loading failed: %s."
                    % (self._adapter_name, str(e)),
                    notify=False,
                    severity="warning",
                )

    def _get_config_from_adapter_configuration(self):
        self.logger.info("Loading config from adapter configuration")
        try:
            adapters = self._client.get_agent_configuration().document.adapters
            for adapter in adapters:
                config = json.loads(adapter.configuration)
                if self._adapter_name in config:
                    return config
        except Exception as e:
            self.logger.warn("Error loading config from adapter config: %s" % str(e))
        return None

    def _get_config_from_blob_data(self):
        self.logger.info("Loading config from blob data")
        try:
            config = json.loads(self._client.get_config_blob_data())
            if self._adapter_name in config:
                return config
        except Exception as e:
            self.logger.info("Error loading config from blob data: %s" % str(e))
        return None

    def _get_config_from_json(self):
        # Try to get config from config.json
        try:
            with open("config.json") as f:
                config = json.loads(f.read())
                if self._adapter_name in config:
                    return config
        except Exception as e:
            self.logger.info("Could not get config from config.json: %s" % str(e))
        return None

    def _validate_schema(self, config_blob):
        # Validate configuration based on schema
        with open("config_schema.json") as f:
            try:
                self.config_schema = json.load(f)
                self.logger.info("Loaded config schema from config_schema.json file")
            except JSONDecodeError as e:
                self.logger.warn(
                    "Could not load config schema. Is the file valid json?"
                )
                raise Exception("config schema error: %s" % str(e))
        self.logger.info("Validating config...")
        try:
            jsonschema.validate(config_blob, self.config_schema)
            self.logger.info("Validation succeeded")
        except (
            jsonschema.ValidationError,
            jsonschema.SchemaError,
            jsonschema.RefResolutionError,
        ) as e:
            self.logger.warn("Validation failed %s: %s", type(e).__name__, str(e))
        except Exception as e:
            self.logger.warn(
                "Validation failed with unkown error %s: %s", type(e).__name__, str(e)
            )



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.filter_types_item import FilterTypesItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="Filter")


@attr.s(auto_attribs=True)
class Filter:
    """
    Attributes:
        agent_ids (Union[Unset, List[str]]):
        device_ids (Union[Unset, List[str]]):
        names (Union[Unset, List[str]]):
        types (Union[Unset, List[FilterTypesItem]]):
        tags (Union[Unset, Any]):
        not_names (Union[Unset, List[str]]):
    """

    agent_ids: Union[Unset, List[str]] = UNSET
    device_ids: Union[Unset, List[str]] = UNSET
    names: Union[Unset, List[str]] = UNSET
    types: Union[Unset, List[FilterTypesItem]] = UNSET
    tags: Union[Unset, Any] = UNSET
    not_names: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        agent_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.agent_ids, Unset):
            agent_ids = self.agent_ids

        device_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.device_ids, Unset):
            device_ids = self.device_ids

        names: Union[Unset, List[str]] = UNSET
        if not isinstance(self.names, Unset):
            names = self.names

        types: Union[Unset, List[str]] = UNSET
        if not isinstance(self.types, Unset):
            types = []
            for types_item_data in self.types:
                types_item = types_item_data.value

                types.append(types_item)

        tags = self.tags
        not_names: Union[Unset, List[str]] = UNSET
        if not isinstance(self.not_names, Unset):
            not_names = self.not_names

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if agent_ids is not UNSET:
            field_dict["agentIds"] = agent_ids
        if device_ids is not UNSET:
            field_dict["deviceIds"] = device_ids
        if names is not UNSET:
            field_dict["names"] = names
        if types is not UNSET:
            field_dict["types"] = types
        if tags is not UNSET:
            field_dict["tags"] = tags
        if not_names is not UNSET:
            field_dict["notNames"] = not_names

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        agent_ids = cast(List[str], d.pop("agentIds", UNSET))

        device_ids = cast(List[str], d.pop("deviceIds", UNSET))

        names = cast(List[str], d.pop("names", UNSET))

        types = []
        _types = d.pop("types", UNSET)
        for types_item_data in _types or []:
            types_item = FilterTypesItem(types_item_data)

            types.append(types_item)

        tags = d.pop("tags", UNSET)

        not_names = cast(List[str], d.pop("notNames", UNSET))

        filter_ = cls(
            agent_ids=agent_ids,
            device_ids=device_ids,
            names=names,
            types=types,
            tags=tags,
            not_names=not_names,
        )

        filter_.additional_properties = d
        return filter_

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import array
import sys
from typing import Iterable
import struct
import lzf
import numpy as np
from .internal_types import PointCloud2, PointField

from formant.protos.model.v1.media_pb2 import PointCloud

# TODO: support different types
FLOAT_DATA_TYPE = 7
DOUBLE_DATA_TYPE = 8


try:
    from numpy.lib.recfunctions import unstructured_to_structured
except ImportError:
    from .numpy_compat import unstructured_to_structured


_DATATYPES = {}
_DATATYPES[PointField.INT8] = np.dtype(np.int8)
_DATATYPES[PointField.UINT8] = np.dtype(np.uint8)
_DATATYPES[PointField.INT16] = np.dtype(np.int16)
_DATATYPES[PointField.UINT16] = np.dtype(np.uint16)
_DATATYPES[PointField.INT32] = np.dtype(np.int32)
_DATATYPES[PointField.UINT32] = np.dtype(np.uint32)
_DATATYPES[PointField.FLOAT32] = np.dtype(np.float32)
_DATATYPES[PointField.FLOAT64] = np.dtype(np.float64)

DUMMY_FIELD_PREFIX = "unnamed_field"


def dtype_from_fields(
    fields,  # type: Iterable[PointField]
):
    # type: (...) -> np.dtype
    """
    Convert a Iterable of sensor_msgs.msg.PointField messages to a np.dtype.
    :param fields: The point cloud fields.
                   (Type: iterable of sensor_msgs.msg.PointField)
    :returns: NumPy datatype
    """
    # Create a lists containing the names, offsets and datatypes of all fields
    field_names = []
    field_offsets = []
    field_datatypes = []
    for i, field in enumerate(fields):
        # Datatype as numpy datatype
        datatype = _DATATYPES[field.datatype]
        # Name field
        if field.name == "":
            name = "%s_%s" % (DUMMY_FIELD_PREFIX, i)
        else:
            name = field.name
        # Handle fields with count > 1 by creating subfields with a suffix consiting
        # of "_" followed by the subfield counter [0 -> (count - 1)]
        assert field.count > 0, "Can't process fields with count = 0."
        for a in range(field.count):
            # Add suffix if we have multiple subfields
            if field.count > 1:
                subfield_name = "%s_%s" % (name, a)
            else:
                subfield_name = name
            assert (
                subfield_name not in field_names
            ), "Duplicate field names are not allowed!"
            field_names.append(subfield_name)
            # Create new offset that includes subfields
            field_offsets.append(field.offset + a * datatype.itemsize)
            field_datatypes.append(datatype.str)

    # Create a tuple for each field containing name and data type
    return np.dtype(
        {"names": field_names, "formats": field_datatypes, "offsets": field_offsets}
    )


def create_cloud(
    fields,  # type: Iterable[PointField]
    points,  # type: Iterable
):
    # type (...)-> PointCloud2
    """
    Create a sensor_msgs.msg.PointCloud2 message.
    :param header: The point cloud header. (Type: std_msgs.msg.Header)
    :param fields: The point cloud fields.
                   (Type: iterable of sensor_msgs.msg.PointField)
    :param points: The point cloud points. List of iterables, i.e. one iterable
                   for each point, with the elements of each iterable being the
                   values of the fields for that point (in the same order as
                   the fields parameter)
    :return: The point cloud as sensor_msgs.msg.PointCloud2
    """
    # Check if input is numpy array
    if isinstance(points, np.ndarray):
        # Check if this is an unstructured array
        if points.dtype.names is None:
            assert all(
                fields[0].datatype == field.datatype for field in fields[1:]
            ), "All fields need to have the same datatype. Pass a structured NumPy array \
                    with multiple dtypes otherwise."
            # Convert unstructured to structured array
            points = unstructured_to_structured(points, dtype=dtype_from_fields(fields))
        else:
            assert points.dtype == dtype_from_fields(
                fields
            ), "PointFields and structured NumPy array dtype do not match for all fields! \
                    Check their field order, names and types."
    else:
        # Cast python objects to structured NumPy array (slow)
        points = np.array(
            # Points need to be tuples in the structured array
            list(map(tuple, points)),
            dtype=dtype_from_fields(fields),
        )

    # Handle organized clouds
    assert (
        len(points.shape) <= 2
    ), "Too many dimensions for organized cloud! \
            Points can only be organized in max. two dimensional space"
    height = 1
    width = points.shape[0]
    # Check if input points are an organized cloud (2D array of points)
    if len(points.shape) == 2:
        height = points.shape[1]

    # Convert numpy points to array.array
    memory_view = memoryview(points)
    casted = memory_view.cast("B")
    array_array = array.array("B")
    array_array.frombytes(casted)

    # Put everything together
    cloud = PointCloud2(
        height=height,
        width=width,
        is_dense=False,
        is_bigendian=sys.byteorder != "little",
        fields=fields,
        point_step=points.dtype.itemsize,
        row_step=(points.dtype.itemsize * width),
    )

    cloud.data = array_array
    return cloud


def pointcloud2_to_formant_pointcloud(
    message,  # type: PointCloud2
):
    if message.is_bigendian:
        raise Exception("unsupported point cloud endianness")
    if message.data is None:
        raise Exception("Pointcloud has no data")
    x_offset = None
    y_offset = None
    z_offset = None
    intensity_offset = None

    x_size = None
    y_size = None
    z_size = None
    intensity_size = None

    for field in message.fields:
        if field.name == "x":
            x_offset = field.offset
            x_size = 8 if field.datatype == DOUBLE_DATA_TYPE else 4
        elif field.name == "y":
            y_offset = field.offset
            y_size = 8 if field.datatype == DOUBLE_DATA_TYPE else 4
        elif field.name == "z":
            z_offset = field.offset
            z_size = 8 if field.datatype == DOUBLE_DATA_TYPE else 4
        elif field.name == "intensity" or field.name == "rgb":
            intensity_offset = field.offset
            intensity_size = 8 if field.datatype == DOUBLE_DATA_TYPE else 4
        else:
            continue

        if field.datatype not in [FLOAT_DATA_TYPE, DOUBLE_DATA_TYPE]:
            raise Exception(
                "error: unsupported pointcloud2 datatype: %s for field: %s"
                % (field.datatype, field.name)
            )

        if field.count != 1:
            raise Exception("error: unsupported pointcloud2 count")

    if (
        x_offset is None
        or y_offset is None
        or z_offset is None
        or x_size is None
        or y_size is None
        or z_size is None
    ):
        raise Exception("Error: Missing X, Y, or Z fields")

    count = message.height * message.width

    xs = np.zeros((count, x_size), dtype="b")
    ys = np.zeros((count, y_size), dtype="b")
    zs = np.zeros((count, z_size), dtype="b")
    if intensity_offset is not None:
        intensities = np.zeros((count, intensity_size), dtype="b")

    if message.point_step:
        size = message.point_step
    else:
        size = 4 * x_size

    data = np.reshape(np.array(message.data, dtype="b"), (count, size),)

    xs = data[:, x_offset : x_offset + x_size]
    ys = data[:, y_offset : y_offset + y_size]
    zs = data[:, z_offset : z_offset + z_size]
    if intensity_offset is not None:
        intensities = data[:, intensity_offset : intensity_offset + intensity_size]

    if intensity_offset is None:
        points = np.concatenate((xs, ys, zs)).tobytes()
    else:
        points = np.concatenate((xs, ys, zs, intensities)).tobytes()

    buffer = (
        """VERSION 0.7
FIELDS x y z rgb
SIZE 4 4 4 4
TYPE F F F F
WIDTH %s
HEIGHT 1
DATA binary_compressed
"""
        % count
    )
    compressed = lzf.compress(points, 2 * len(points))
    raw = (
        buffer.encode("utf-8")
        + struct.pack("<I", len(compressed))
        + struct.pack("<I", len(points))
        + compressed
    )

    return raw



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.count_history_query_type import CountHistoryQueryType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CountHistoryQuery")


@attr.s(auto_attribs=True)
class CountHistoryQuery:
    """
    Attributes:
        type (CountHistoryQueryType):
        start (datetime.datetime):
        end (datetime.datetime):
        organization_id (Union[Unset, str]):
    """

    type: CountHistoryQueryType
    start: datetime.datetime
    end: datetime.datetime
    organization_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        start = self.start.isoformat()

        end = self.end.isoformat()

        organization_id = self.organization_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "start": start,
                "end": end,
            }
        )
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = CountHistoryQueryType(d.pop("type"))

        start = isoparse(d.pop("start"))

        end = isoparse(d.pop("end"))

        organization_id = d.pop("organizationId", UNSET)

        count_history_query = cls(
            type=type,
            start=start,
            end=end,
            organization_id=organization_id,
        )

        count_history_query.additional_properties = d
        return count_history_query

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ForwardingConfiguration")


@attr.s(auto_attribs=True)
class ForwardingConfiguration:
    """
    Attributes:
        pagerduty (Union[Unset, bool]):
        slack (Union[Unset, List[Any]]):
        webhooks (Union[Unset, List[str]]):
        sms (Union[Unset, bool]):
    """

    pagerduty: Union[Unset, bool] = UNSET
    slack: Union[Unset, List[Any]] = UNSET
    webhooks: Union[Unset, List[str]] = UNSET
    sms: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        pagerduty = self.pagerduty
        slack: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.slack, Unset):
            slack = self.slack

        webhooks: Union[Unset, List[str]] = UNSET
        if not isinstance(self.webhooks, Unset):
            webhooks = self.webhooks

        sms = self.sms

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if pagerduty is not UNSET:
            field_dict["pagerduty"] = pagerduty
        if slack is not UNSET:
            field_dict["slack"] = slack
        if webhooks is not UNSET:
            field_dict["webhooks"] = webhooks
        if sms is not UNSET:
            field_dict["sms"] = sms

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        pagerduty = d.pop("pagerduty", UNSET)

        slack = cast(List[Any], d.pop("slack", UNSET))

        webhooks = cast(List[str], d.pop("webhooks", UNSET))

        sms = d.pop("sms", UNSET)

        forwarding_configuration = cls(
            pagerduty=pagerduty,
            slack=slack,
            webhooks=webhooks,
            sms=sms,
        )

        forwarding_configuration.additional_properties = d
        return forwarding_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class BaseEventType(str, Enum):
    TRIGGERED_EVENT = "triggered-event"
    DATAPOINT_EVENT = "datapoint-event"
    DEVICE_ONLINE = "device-online"
    DEVICE_OFFLINE = "device-offline"
    INTERVENTION_REQUEST = "intervention-request"
    TELEOP_SESSION_RECORD = "teleop-session-record"
    PORT_FORWARDING_SESSION_RECORD = "port-forwarding-session-record"
    COMMAND_REQUEST = "command-request"
    COMMAND_RESPONSE = "command-response"
    COMMAND_DELIVERY = "command-delivery"
    CUSTOM = "custom"
    COMMENT = "comment"
    SYSTEM = "system"
    ANNOTATION = "annotation"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.capture_session_tags import CaptureSessionTags


T = TypeVar("T", bound="CaptureSession")


@attr.s(auto_attribs=True)
class CaptureSession:
    """
    Attributes:
        device_id (str):
        stream_name (str):
        tags (CaptureSessionTags):
        organization_id (Union[Unset, str]):
        user_id (Union[Unset, str]):
        code (Union[Unset, str]):
        expiration (Union[Unset, datetime.datetime]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
    """

    device_id: str
    stream_name: str
    tags: "CaptureSessionTags"
    organization_id: Union[Unset, str] = UNSET
    user_id: Union[Unset, str] = UNSET
    code: Union[Unset, str] = UNSET
    expiration: Union[Unset, datetime.datetime] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        device_id = self.device_id
        stream_name = self.stream_name
        tags = self.tags.to_dict()

        organization_id = self.organization_id
        user_id = self.user_id
        code = self.code
        expiration: Union[Unset, str] = UNSET
        if not isinstance(self.expiration, Unset):
            expiration = self.expiration.isoformat()

        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "deviceId": device_id,
                "streamName": stream_name,
                "tags": tags,
            }
        )
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if user_id is not UNSET:
            field_dict["userId"] = user_id
        if code is not UNSET:
            field_dict["code"] = code
        if expiration is not UNSET:
            field_dict["expiration"] = expiration
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.capture_session_tags import CaptureSessionTags

        d = src_dict.copy()
        device_id = d.pop("deviceId")

        stream_name = d.pop("streamName")

        tags = CaptureSessionTags.from_dict(d.pop("tags"))

        organization_id = d.pop("organizationId", UNSET)

        user_id = d.pop("userId", UNSET)

        code = d.pop("code", UNSET)

        _expiration = d.pop("expiration", UNSET)
        expiration: Union[Unset, datetime.datetime]
        if isinstance(_expiration, Unset):
            expiration = UNSET
        else:
            expiration = isoparse(_expiration)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        capture_session = cls(
            device_id=device_id,
            stream_name=stream_name,
            tags=tags,
            organization_id=organization_id,
            user_id=user_id,
            code=code,
            expiration=expiration,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
        )

        capture_session.additional_properties = d
        return capture_session

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_tags import UserTags


T = TypeVar("T", bound="User")


@attr.s(auto_attribs=True)
class User:
    """
    Attributes:
        email (str):
        first_name (str):
        role_id (str):
        organization_id (Union[Unset, str]):
        last_name (Union[Unset, str]):
        team_id (Union[Unset, None, str]):
        phone_number (Union[Unset, str]):
        enabled (Union[Unset, bool]):
        is_organization_owner (Union[Unset, bool]):
        terms_accepted (Union[Unset, str]):
        last_logged_in (Union[Unset, datetime.datetime]):
        password_hash (Union[Unset, None, str]):
        is_single_sign_on (Union[Unset, bool]):
        is_service_account (Union[Unset, bool]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        tags (Union[Unset, UserTags]):
    """

    email: str
    first_name: str
    role_id: str
    organization_id: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    team_id: Union[Unset, None, str] = UNSET
    phone_number: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    is_organization_owner: Union[Unset, bool] = UNSET
    terms_accepted: Union[Unset, str] = UNSET
    last_logged_in: Union[Unset, datetime.datetime] = UNSET
    password_hash: Union[Unset, None, str] = UNSET
    is_single_sign_on: Union[Unset, bool] = UNSET
    is_service_account: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    tags: Union[Unset, "UserTags"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        email = self.email
        first_name = self.first_name
        role_id = self.role_id
        organization_id = self.organization_id
        last_name = self.last_name
        team_id = self.team_id
        phone_number = self.phone_number
        enabled = self.enabled
        is_organization_owner = self.is_organization_owner
        terms_accepted = self.terms_accepted
        last_logged_in: Union[Unset, str] = UNSET
        if not isinstance(self.last_logged_in, Unset):
            last_logged_in = self.last_logged_in.isoformat()

        password_hash = self.password_hash
        is_single_sign_on = self.is_single_sign_on
        is_service_account = self.is_service_account
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        tags: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "email": email,
                "firstName": first_name,
                "roleId": role_id,
            }
        )
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if team_id is not UNSET:
            field_dict["teamId"] = team_id
        if phone_number is not UNSET:
            field_dict["phoneNumber"] = phone_number
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if is_organization_owner is not UNSET:
            field_dict["isOrganizationOwner"] = is_organization_owner
        if terms_accepted is not UNSET:
            field_dict["termsAccepted"] = terms_accepted
        if last_logged_in is not UNSET:
            field_dict["lastLoggedIn"] = last_logged_in
        if password_hash is not UNSET:
            field_dict["passwordHash"] = password_hash
        if is_single_sign_on is not UNSET:
            field_dict["isSingleSignOn"] = is_single_sign_on
        if is_service_account is not UNSET:
            field_dict["isServiceAccount"] = is_service_account
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_tags import UserTags

        d = src_dict.copy()
        email = d.pop("email")

        first_name = d.pop("firstName")

        role_id = d.pop("roleId")

        organization_id = d.pop("organizationId", UNSET)

        last_name = d.pop("lastName", UNSET)

        team_id = d.pop("teamId", UNSET)

        phone_number = d.pop("phoneNumber", UNSET)

        enabled = d.pop("enabled", UNSET)

        is_organization_owner = d.pop("isOrganizationOwner", UNSET)

        terms_accepted = d.pop("termsAccepted", UNSET)

        _last_logged_in = d.pop("lastLoggedIn", UNSET)
        last_logged_in: Union[Unset, datetime.datetime]
        if isinstance(_last_logged_in, Unset):
            last_logged_in = UNSET
        else:
            last_logged_in = isoparse(_last_logged_in)

        password_hash = d.pop("passwordHash", UNSET)

        is_single_sign_on = d.pop("isSingleSignOn", UNSET)

        is_service_account = d.pop("isServiceAccount", UNSET)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, UserTags]
        if isinstance(_tags, Unset):
            tags = UNSET
        else:
            tags = UserTags.from_dict(_tags)

        user = cls(
            email=email,
            first_name=first_name,
            role_id=role_id,
            organization_id=organization_id,
            last_name=last_name,
            team_id=team_id,
            phone_number=phone_number,
            enabled=enabled,
            is_organization_owner=is_organization_owner,
            terms_accepted=terms_accepted,
            last_logged_in=last_logged_in,
            password_hash=password_hash,
            is_single_sign_on=is_single_sign_on,
            is_service_account=is_service_account,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            tags=tags,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import AnnotationTemplateListResponse, AnnotationStreamType, AnnotationType, AnnotationFieldValuesRequestTags, AnnotationTemplate, AnnotationField, UserParametersRolesItem, UserParameters, AnnotationFieldType, AnnotationFieldValue, TagParameters, AnnotationFieldValuesResponse, Annotation, SheetParameters, AnnotationTemplateTags, AnnotationTags, AnnotationFieldValuesRequest
from formant.sdk.cloud.v2.formant_admin_api_client.api.annotation import annotation_controller_post
from formant.sdk.cloud.v2.formant_admin_api_client.api.annotation_template import annotation_template_controller_list, annotation_template_controller_get_one
from formant.sdk.cloud.v2.formant_admin_api_client.types import *



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.command_parameter import CommandParameter


T = TypeVar("T", bound="CommandRequest")


@attr.s(auto_attribs=True)
class CommandRequest:
    """
    Attributes:
        id (str):
        command (str):
        parameter (CommandParameter):
        created_at (datetime.datetime):
    """

    id: str
    command: str
    parameter: "CommandParameter"
    created_at: datetime.datetime
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        command = self.command
        parameter = self.parameter.to_dict()

        created_at = self.created_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "command": command,
                "parameter": parameter,
                "createdAt": created_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.command_parameter import CommandParameter

        d = src_dict.copy()
        id = d.pop("id")

        command = d.pop("command")

        parameter = CommandParameter.from_dict(d.pop("parameter"))

        created_at = isoparse(d.pop("createdAt"))

        command_request = cls(
            id=id,
            command=command,
            parameter=parameter,
            created_at=created_at,
        )

        command_request.additional_properties = d
        return command_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class IntervalQueryInterval(str, Enum):
    MINUTE = "minute"
    HOUR = "hour"
    DAY = "day"
    MONTH = "month"
    YEAR = "year"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.device_disk_configuration import DeviceDiskConfiguration


T = TypeVar("T", bound="DeviceResourcesConfiguration")


@attr.s(auto_attribs=True)
class DeviceResourcesConfiguration:
    """
    Attributes:
        disk (Union[Unset, DeviceDiskConfiguration]):
        stream_throttle_hz (Union[Unset, None, float]):
        low_bandwidth_agent (Union[Unset, None, bool]):
    """

    disk: Union[Unset, "DeviceDiskConfiguration"] = UNSET
    stream_throttle_hz: Union[Unset, None, float] = UNSET
    low_bandwidth_agent: Union[Unset, None, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        disk: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.disk, Unset):
            disk = self.disk.to_dict()

        stream_throttle_hz = self.stream_throttle_hz
        low_bandwidth_agent = self.low_bandwidth_agent

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if disk is not UNSET:
            field_dict["disk"] = disk
        if stream_throttle_hz is not UNSET:
            field_dict["streamThrottleHz"] = stream_throttle_hz
        if low_bandwidth_agent is not UNSET:
            field_dict["lowBandwidthAgent"] = low_bandwidth_agent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.device_disk_configuration import DeviceDiskConfiguration

        d = src_dict.copy()
        _disk = d.pop("disk", UNSET)
        disk: Union[Unset, DeviceDiskConfiguration]
        if isinstance(_disk, Unset):
            disk = UNSET
        else:
            disk = DeviceDiskConfiguration.from_dict(_disk)

        stream_throttle_hz = d.pop("streamThrottleHz", UNSET)

        low_bandwidth_agent = d.pop("lowBandwidthAgent", UNSET)

        device_resources_configuration = cls(
            disk=disk,
            stream_throttle_hz=stream_throttle_hz,
            low_bandwidth_agent=low_bandwidth_agent,
        )

        device_resources_configuration.additional_properties = d
        return device_resources_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.get_features_response_features_item import GetFeaturesResponseFeaturesItem

T = TypeVar("T", bound="GetFeaturesResponse")


@attr.s(auto_attribs=True)
class GetFeaturesResponse:
    """
    Attributes:
        features (List[GetFeaturesResponseFeaturesItem]):
    """

    features: List[GetFeaturesResponseFeaturesItem]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        features = []
        for features_item_data in self.features:
            features_item = features_item_data.value

            features.append(features_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "features": features,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        features = []
        _features = d.pop("features")
        for features_item_data in _features:
            features_item = GetFeaturesResponseFeaturesItem(features_item_data)

            features.append(features_item)

        get_features_response = cls(
            features=features,
        )

        get_features_response.additional_properties = d
        return get_features_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.usage_prices import UsagePrices


T = TypeVar("T", bound="BillingInfo")


@attr.s(auto_attribs=True)
class BillingInfo:
    """
    Attributes:
        usage_prices (UsagePrices):
        organization_budget_usd_cents (int):
        device_budget_usd_cents (int):
    """

    usage_prices: "UsagePrices"
    organization_budget_usd_cents: int
    device_budget_usd_cents: int
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        usage_prices = self.usage_prices.to_dict()

        organization_budget_usd_cents = self.organization_budget_usd_cents
        device_budget_usd_cents = self.device_budget_usd_cents

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "usagePrices": usage_prices,
                "organizationBudgetUsdCents": organization_budget_usd_cents,
                "deviceBudgetUsdCents": device_budget_usd_cents,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.usage_prices import UsagePrices

        d = src_dict.copy()
        usage_prices = UsagePrices.from_dict(d.pop("usagePrices"))

        organization_budget_usd_cents = d.pop("organizationBudgetUsdCents")

        device_budget_usd_cents = d.pop("deviceBudgetUsdCents")

        billing_info = cls(
            usage_prices=usage_prices,
            organization_budget_usd_cents=organization_budget_usd_cents,
            device_budget_usd_cents=device_budget_usd_cents,
        )

        billing_info.additional_properties = d
        return billing_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.event_counts import EventCounts


T = TypeVar("T", bound="EventHistogramEntry")


@attr.s(auto_attribs=True)
class EventHistogramEntry:
    """
    Attributes:
        time (datetime.datetime):
        counts (EventCounts):
    """

    time: datetime.datetime
    counts: "EventCounts"
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        time = self.time.isoformat()

        counts = self.counts.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "time": time,
                "counts": counts,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.event_counts import EventCounts

        d = src_dict.copy()
        time = isoparse(d.pop("time"))

        counts = EventCounts.from_dict(d.pop("counts"))

        event_histogram_entry = cls(
            time=time,
            counts=counts,
        )

        event_histogram_entry.additional_properties = d
        return event_histogram_entry

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="OnDemandPresenceTimeRange")


@attr.s(auto_attribs=True)
class OnDemandPresenceTimeRange:
    """
    Attributes:
        start (int):
        end (int):
        byte_size (float):
        item_count (float):
    """

    start: int
    end: int
    byte_size: float
    item_count: float
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        start = self.start
        end = self.end
        byte_size = self.byte_size
        item_count = self.item_count

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "start": start,
                "end": end,
                "byteSize": byte_size,
                "itemCount": item_count,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start = d.pop("start")

        end = d.pop("end")

        byte_size = d.pop("byteSize")

        item_count = d.pop("itemCount")

        on_demand_presence_time_range = cls(
            start=start,
            end=end,
            byte_size=byte_size,
            item_count=item_count,
        )

        on_demand_presence_time_range.additional_properties = d
        return on_demand_presence_time_range

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ThresholdEventTriggerConditionOperator(str, Enum):
    VALUE_0 = "<"
    VALUE_1 = "<="
    VALUE_2 = ">"
    VALUE_3 = ">="
    VALUE_4 = "=="
    VALUE_5 = "!="

    def __str__(self) -> str:
        return str(self.value)



import unittest
from formant.sdk.cloud.v2.src.admin_api import AdminAPI
from formant.sdk.cloud.v2.formant_admin_api_client.models import (
    Command,
    CommandParameter,
    CommandQuery,
)
from dateutil import parser
import os

EMAIL = os.getenv("FORMANT_EMAIL")
PASSWORD = os.getenv("FORMANT_PASSWORD")
BASE_URL = "https://api.formant.io/v1"


class TestCommands(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestCommands, self).__init__(*args, **kwargs)
        self.client = AdminAPI(email=EMAIL, password=PASSWORD, base_url=BASE_URL)

    def test_create_command(self):
        device_id = "404e2d2c-f95f-41d8-bce1-915e314a6898"
        command_name = "return_to_charge_station"
        scrubber_time = parser.parse("2014-11-03T19:38:34.203Z")
        parameter = CommandParameter(scrubber_time=scrubber_time)
        command = Command(
            device_id=device_id,
            command=command_name,
            parameter=parameter,
            organization_id=self.client.organization_id,
        )
        result = self.client.commands.create(command)
        self.assertEqual(result.status_code, 201)

    def test_query_commands(self):
        device_id = "404e2d2c-f95f-41d8-bce1-915e314a6898"
        command_query = CommandQuery(device_id=device_id)
        result = self.client.commands.query(command_query)
        self.assertEqual(result.status_code, 200)


unittest.main()



from http import HTTPStatus
from typing import Any, Dict

import httpx

from ...client import AuthenticatedClient
from ...types import Response


def _get_kwargs(
    id: str,
    *,
    client: AuthenticatedClient,
) -> Dict[str, Any]:
    url = "{}/devices/{id}/unprovision".format(client.base_url, id=id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Any]:
    """Unprovision device

     Unprovision a device
    Resource: devices
    Authorized roles: administrator

    Args:
        id (str):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Any]:
    """Unprovision device

     Unprovision a device
    Resource: devices
    Authorized roles: administrator

    Args:
        id (str):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



import unittest
from formant.sdk.cloud.v2.src.query_api import QueryAPI
from formant.sdk.cloud.v2.formant_query_api_client.models.query import (
    Query,
    QueryTypesItem,
)
from formant.sdk.cloud.v2.formant_query_api_client.models.stream_data_points_item import (
    StreamDataPointsItem,
)
import dateutil.parser as parser
import os

EMAIL = os.getenv("FORMANT_EMAIL")
PASSWORD = os.getenv("FORMANT_PASSWORD")
BASE_URL = "https://api.formant.io/v1"


class TestQueries(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestQueries, self).__init__(*args, **kwargs)
        self.client = QueryAPI(email=EMAIL, password=PASSWORD, base_url=BASE_URL)

    def test_query_numeric(self):
        start = parser.isoparse("2022-11-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.NUMERIC]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)

        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.numeric, True)

    def test_query_text(self):
        start = parser.isoparse("2022-11-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.TEXT]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.text, True)

    def test_query_bitset(self):
        start = parser.isoparse("2022-11-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.BITSET]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)

        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.bitset, True)

    def test_query_localization(self):
        start = parser.isoparse("2022-10-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.LOCALIZATION]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.localization, True)

    def test_query_point_cloud(self):
        start = parser.isoparse("2022-08-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.POINT_CLOUD]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.point_cloud, True)

    def test_query_location(self):
        start = parser.isoparse("2022-06-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.LOCATION]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.location, True)

    def test_query_file(self):
        start = parser.isoparse("2022-11-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.FILE]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.file, True)

    def test_query_health(self):
        start = parser.isoparse("2022-11-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.HEALTH]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.health, True)

    def test_query_transform_tree(self):
        start = parser.isoparse("2022-11-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.TRANSFORM_TREE]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.transform_tree, True)

    def test_query_battery(self):
        start = parser.isoparse("2022-11-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.BATTERY]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.battery, True)

    def test_query_video(self):
        start = parser.isoparse("2022-11-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.VIDEO]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.video, True)

    def test_query_numeric_set(self):
        start = parser.isoparse("2022-11-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.NUMERIC_SET]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.numeric_set, True)

    def test_query_json(self):
        start = parser.isoparse("2022-11-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.JSON]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.json, True)

    def test_query_image(self):
        start = parser.isoparse("2022-11-11T02:00:00.000Z")
        end = parser.isoparse("2022-12-11T02:02:00.000Z")

        types = [QueryTypesItem.IMAGE]
        query = Query(start=start, end=end, types=types, next_="true")
        result = self.client.queries.query(query=query)

        self.assertEqual(result.status_code, 200)
        stream_data_points_item = result.parsed.items[0].points[0]

        self.assertIsInstance(
            stream_data_points_item,
            StreamDataPointsItem,
            "not an instance of StreamDataPointsItem",
        )

        self.assertIsNotNone(stream_data_points_item.image, True)


unittest.main()



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="BoundingBox")


@attr.s(auto_attribs=True)
class BoundingBox:
    """
    Attributes:
        x (int):
        y (int):
        width (int):
        height (int):
    """

    x: int
    y: int
    width: int
    height: int
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        x = self.x
        y = self.y
        width = self.width
        height = self.height

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "x": x,
                "y": y,
                "width": width,
                "height": height,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        x = d.pop("x")

        y = d.pop("y")

        width = d.pop("width")

        height = d.pop("height")

        bounding_box = cls(
            x=x,
            y=y,
            width=width,
            height=height,
        )

        bounding_box.additional_properties = d
        return bounding_box

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# flake8: noqa
from .goal import Goal
from .odometry import Odometry
from .path import Path
from .quaternion import Quaternion
from .transform import Transform
from .twist import Twist
from .vector3 import Vector3
from .map import Map
from .point_cloud import PointCloud



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.transform import Transform


T = TypeVar("T", bound="Map")


@attr.s(auto_attribs=True)
class Map:
    """
    Attributes:
        url (str):
        width (int):
        height (int):
        resolution (float):
        origin (Transform):
        world_to_local (Transform):
        size (Union[Unset, int]):
    """

    url: str
    width: int
    height: int
    resolution: float
    origin: "Transform"
    world_to_local: "Transform"
    size: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        url = self.url
        width = self.width
        height = self.height
        resolution = self.resolution
        origin = self.origin.to_dict()

        world_to_local = self.world_to_local.to_dict()

        size = self.size

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "url": url,
                "width": width,
                "height": height,
                "resolution": resolution,
                "origin": origin,
                "worldToLocal": world_to_local,
            }
        )
        if size is not UNSET:
            field_dict["size"] = size

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.transform import Transform

        d = src_dict.copy()
        url = d.pop("url")

        width = d.pop("width")

        height = d.pop("height")

        resolution = d.pop("resolution")

        origin = Transform.from_dict(d.pop("origin"))

        world_to_local = Transform.from_dict(d.pop("worldToLocal"))

        size = d.pop("size", UNSET)

        map_ = cls(
            url=url,
            width=width,
            height=height,
            resolution=resolution,
            origin=origin,
            world_to_local=world_to_local,
            size=size,
        )

        map_.additional_properties = d
        return map_

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.triggered_event_severity import TriggeredEventSeverity
from ..models.triggered_event_stream_type import TriggeredEventStreamType
from ..models.triggered_event_type import TriggeredEventType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.triggered_event_tags import TriggeredEventTags


T = TypeVar("T", bound="TriggeredEvent")


@attr.s(auto_attribs=True)
class TriggeredEvent:
    """
    Attributes:
        severity (TriggeredEventSeverity):
        event_trigger_id (str):
        interval (int):
        time (datetime.datetime):
        interval_start (Union[Unset, datetime.datetime]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        type (Union[Unset, TriggeredEventType]):
        organization_id (Union[Unset, str]):
        end_time (Union[Unset, None, datetime.datetime]):
        message (Union[Unset, str]):
        viewed (Union[Unset, bool]):
        device_id (Union[Unset, None, str]):
        stream_name (Union[Unset, None, str]):
        stream_type (Union[Unset, None, TriggeredEventStreamType]):
        tags (Union[Unset, None, TriggeredEventTags]):
        notification_enabled (Union[Unset, bool]):
    """

    severity: TriggeredEventSeverity
    event_trigger_id: str
    interval: int
    time: datetime.datetime
    interval_start: Union[Unset, datetime.datetime] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    type: Union[Unset, TriggeredEventType] = UNSET
    organization_id: Union[Unset, str] = UNSET
    end_time: Union[Unset, None, datetime.datetime] = UNSET
    message: Union[Unset, str] = UNSET
    viewed: Union[Unset, bool] = UNSET
    device_id: Union[Unset, None, str] = UNSET
    stream_name: Union[Unset, None, str] = UNSET
    stream_type: Union[Unset, None, TriggeredEventStreamType] = UNSET
    tags: Union[Unset, None, "TriggeredEventTags"] = UNSET
    notification_enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        severity = self.severity.value

        event_trigger_id = self.event_trigger_id
        interval = self.interval
        time = self.time.isoformat()

        interval_start: Union[Unset, str] = UNSET
        if not isinstance(self.interval_start, Unset):
            interval_start = self.interval_start.isoformat()

        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        organization_id = self.organization_id
        end_time: Union[Unset, None, str] = UNSET
        if not isinstance(self.end_time, Unset):
            end_time = self.end_time.isoformat() if self.end_time else None

        message = self.message
        viewed = self.viewed
        device_id = self.device_id
        stream_name = self.stream_name
        stream_type: Union[Unset, None, str] = UNSET
        if not isinstance(self.stream_type, Unset):
            stream_type = self.stream_type.value if self.stream_type else None

        tags: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict() if self.tags else None

        notification_enabled = self.notification_enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "severity": severity,
                "eventTriggerId": event_trigger_id,
                "interval": interval,
                "time": time,
            }
        )
        if interval_start is not UNSET:
            field_dict["intervalStart"] = interval_start
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if type is not UNSET:
            field_dict["type"] = type
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if end_time is not UNSET:
            field_dict["endTime"] = end_time
        if message is not UNSET:
            field_dict["message"] = message
        if viewed is not UNSET:
            field_dict["viewed"] = viewed
        if device_id is not UNSET:
            field_dict["deviceId"] = device_id
        if stream_name is not UNSET:
            field_dict["streamName"] = stream_name
        if stream_type is not UNSET:
            field_dict["streamType"] = stream_type
        if tags is not UNSET:
            field_dict["tags"] = tags
        if notification_enabled is not UNSET:
            field_dict["notificationEnabled"] = notification_enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.triggered_event_tags import TriggeredEventTags

        d = src_dict.copy()
        severity = TriggeredEventSeverity(d.pop("severity"))

        event_trigger_id = d.pop("eventTriggerId")

        interval = d.pop("interval")

        time = isoparse(d.pop("time"))

        _interval_start = d.pop("intervalStart", UNSET)
        interval_start: Union[Unset, datetime.datetime]
        if isinstance(_interval_start, Unset):
            interval_start = UNSET
        else:
            interval_start = isoparse(_interval_start)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _type = d.pop("type", UNSET)
        type: Union[Unset, TriggeredEventType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = TriggeredEventType(_type)

        organization_id = d.pop("organizationId", UNSET)

        _end_time = d.pop("endTime", UNSET)
        end_time: Union[Unset, None, datetime.datetime]
        if _end_time is None:
            end_time = None
        elif isinstance(_end_time, Unset):
            end_time = UNSET
        else:
            end_time = isoparse(_end_time)

        message = d.pop("message", UNSET)

        viewed = d.pop("viewed", UNSET)

        device_id = d.pop("deviceId", UNSET)

        stream_name = d.pop("streamName", UNSET)

        _stream_type = d.pop("streamType", UNSET)
        stream_type: Union[Unset, None, TriggeredEventStreamType]
        if _stream_type is None:
            stream_type = None
        elif isinstance(_stream_type, Unset):
            stream_type = UNSET
        else:
            stream_type = TriggeredEventStreamType(_stream_type)

        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, None, TriggeredEventTags]
        if _tags is None:
            tags = None
        elif isinstance(_tags, Unset):
            tags = UNSET
        else:
            tags = TriggeredEventTags.from_dict(_tags)

        notification_enabled = d.pop("notificationEnabled", UNSET)

        triggered_event = cls(
            severity=severity,
            event_trigger_id=event_trigger_id,
            interval=interval,
            time=time,
            interval_start=interval_start,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            type=type,
            organization_id=organization_id,
            end_time=end_time,
            message=message,
            viewed=viewed,
            device_id=device_id,
            stream_name=stream_name,
            stream_type=stream_type,
            tags=tags,
            notification_enabled=notification_enabled,
        )

        triggered_event.additional_properties = d
        return triggered_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="Module")


@attr.s(auto_attribs=True)
class Module:
    """
    Attributes:
        name (str):
        url (str):
        organization_id (Union[Unset, str]):
        preview_image_url (Union[Unset, str]):
        icon_image_url (Union[Unset, None, str]):
        configuration_schema_url (Union[Unset, None, str]):
        description (Union[Unset, None, str]):
        help_url (Union[Unset, None, str]):
        fullscreen (Union[Unset, bool]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
    """

    name: str
    url: str
    organization_id: Union[Unset, str] = UNSET
    preview_image_url: Union[Unset, str] = UNSET
    icon_image_url: Union[Unset, None, str] = UNSET
    configuration_schema_url: Union[Unset, None, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    help_url: Union[Unset, None, str] = UNSET
    fullscreen: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        url = self.url
        organization_id = self.organization_id
        preview_image_url = self.preview_image_url
        icon_image_url = self.icon_image_url
        configuration_schema_url = self.configuration_schema_url
        description = self.description
        help_url = self.help_url
        fullscreen = self.fullscreen
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "url": url,
            }
        )
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if preview_image_url is not UNSET:
            field_dict["previewImageUrl"] = preview_image_url
        if icon_image_url is not UNSET:
            field_dict["iconImageUrl"] = icon_image_url
        if configuration_schema_url is not UNSET:
            field_dict["configurationSchemaUrl"] = configuration_schema_url
        if description is not UNSET:
            field_dict["description"] = description
        if help_url is not UNSET:
            field_dict["helpUrl"] = help_url
        if fullscreen is not UNSET:
            field_dict["fullscreen"] = fullscreen
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        url = d.pop("url")

        organization_id = d.pop("organizationId", UNSET)

        preview_image_url = d.pop("previewImageUrl", UNSET)

        icon_image_url = d.pop("iconImageUrl", UNSET)

        configuration_schema_url = d.pop("configurationSchemaUrl", UNSET)

        description = d.pop("description", UNSET)

        help_url = d.pop("helpUrl", UNSET)

        fullscreen = d.pop("fullscreen", UNSET)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        module = cls(
            name=name,
            url=url,
            organization_id=organization_id,
            preview_image_url=preview_image_url,
            icon_image_url=icon_image_url,
            configuration_schema_url=configuration_schema_url,
            description=description,
            help_url=help_url,
            fullscreen=fullscreen,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
        )

        module.additional_properties = d
        return module

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import BeginUploadRequest, BeginUploadResponse, CompleteUploadRequest
from formant.sdk.cloud.v2.formant_admin_api_client.api.file import file_controller_begin_upload, file_controller_complete_upload
from formant.sdk.cloud.v2.formant_admin_api_client.types import *
from typing import Any



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_teleop_twist_ros_topic_configuration import UserTeleopTwistRosTopicConfiguration


T = TypeVar("T", bound="UserTeleopRosStreamConfiguration")


@attr.s(auto_attribs=True)
class UserTeleopRosStreamConfiguration:
    """
    Attributes:
        topic_name (str):
        display_name (Union[Unset, str]):
        configuration (Union[Unset, UserTeleopTwistRosTopicConfiguration]):
    """

    topic_name: str
    display_name: Union[Unset, str] = UNSET
    configuration: Union[Unset, "UserTeleopTwistRosTopicConfiguration"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        topic_name = self.topic_name
        display_name = self.display_name
        configuration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.configuration, Unset):
            configuration = self.configuration.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "topicName": topic_name,
            }
        )
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if configuration is not UNSET:
            field_dict["configuration"] = configuration

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_teleop_twist_ros_topic_configuration import UserTeleopTwistRosTopicConfiguration

        d = src_dict.copy()
        topic_name = d.pop("topicName")

        display_name = d.pop("displayName", UNSET)

        _configuration = d.pop("configuration", UNSET)
        configuration: Union[Unset, UserTeleopTwistRosTopicConfiguration]
        if isinstance(_configuration, Unset):
            configuration = UNSET
        else:
            configuration = UserTeleopTwistRosTopicConfiguration.from_dict(_configuration)

        user_teleop_ros_stream_configuration = cls(
            topic_name=topic_name,
            display_name=display_name,
            configuration=configuration,
        )

        user_teleop_ros_stream_configuration.additional_properties = d
        return user_teleop_ros_stream_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.device_configuration_document import DeviceConfigurationDocument


T = TypeVar("T", bound="DeviceConfiguration")


@attr.s(auto_attribs=True)
class DeviceConfiguration:
    """
    Attributes:
        document (DeviceConfigurationDocument):
        device_id (Union[Unset, str]):
        version (Union[Unset, int]):
        created_at (Union[Unset, datetime.datetime]):
    """

    document: "DeviceConfigurationDocument"
    device_id: Union[Unset, str] = UNSET
    version: Union[Unset, int] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        document = self.document.to_dict()

        device_id = self.device_id
        version = self.version
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "document": document,
            }
        )
        if device_id is not UNSET:
            field_dict["deviceId"] = device_id
        if version is not UNSET:
            field_dict["version"] = version
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.device_configuration_document import DeviceConfigurationDocument

        d = src_dict.copy()
        document = DeviceConfigurationDocument.from_dict(d.pop("document"))

        device_id = d.pop("deviceId", UNSET)

        version = d.pop("version", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        device_configuration = cls(
            document=document,
            device_id=device_id,
            version=version,
            created_at=created_at,
        )

        device_configuration.additional_properties = d
        return device_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.device_stream_hardware_configuration_hardware_type import DeviceStreamHardwareConfigurationHardwareType
from ..models.device_stream_hardware_configuration_quality import DeviceStreamHardwareConfigurationQuality
from ..models.device_stream_hardware_configuration_type import DeviceStreamHardwareConfigurationType
from ..types import UNSET, Unset

T = TypeVar("T", bound="DeviceStreamHardwareConfiguration")


@attr.s(auto_attribs=True)
class DeviceStreamHardwareConfiguration:
    """
    Attributes:
        type (DeviceStreamHardwareConfigurationType):
        hw_descriptor (str):
        hardware_type (DeviceStreamHardwareConfigurationHardwareType):
        audio_hw_descriptor (Union[Unset, str]):
        rtsp_encoding_needed (Union[Unset, bool]):
        is_onvif (Union[Unset, bool]):
        ip_cam_username (Union[Unset, str]):
        ip_cam_password (Union[Unset, str]):
        quality (Union[Unset, DeviceStreamHardwareConfigurationQuality]):
    """

    type: DeviceStreamHardwareConfigurationType
    hw_descriptor: str
    hardware_type: DeviceStreamHardwareConfigurationHardwareType
    audio_hw_descriptor: Union[Unset, str] = UNSET
    rtsp_encoding_needed: Union[Unset, bool] = UNSET
    is_onvif: Union[Unset, bool] = UNSET
    ip_cam_username: Union[Unset, str] = UNSET
    ip_cam_password: Union[Unset, str] = UNSET
    quality: Union[Unset, DeviceStreamHardwareConfigurationQuality] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        hw_descriptor = self.hw_descriptor
        hardware_type = self.hardware_type.value

        audio_hw_descriptor = self.audio_hw_descriptor
        rtsp_encoding_needed = self.rtsp_encoding_needed
        is_onvif = self.is_onvif
        ip_cam_username = self.ip_cam_username
        ip_cam_password = self.ip_cam_password
        quality: Union[Unset, str] = UNSET
        if not isinstance(self.quality, Unset):
            quality = self.quality.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "hwDescriptor": hw_descriptor,
                "hardwareType": hardware_type,
            }
        )
        if audio_hw_descriptor is not UNSET:
            field_dict["audioHwDescriptor"] = audio_hw_descriptor
        if rtsp_encoding_needed is not UNSET:
            field_dict["rtspEncodingNeeded"] = rtsp_encoding_needed
        if is_onvif is not UNSET:
            field_dict["isOnvif"] = is_onvif
        if ip_cam_username is not UNSET:
            field_dict["ipCamUsername"] = ip_cam_username
        if ip_cam_password is not UNSET:
            field_dict["ipCamPassword"] = ip_cam_password
        if quality is not UNSET:
            field_dict["quality"] = quality

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = DeviceStreamHardwareConfigurationType(d.pop("type"))

        hw_descriptor = d.pop("hwDescriptor")

        hardware_type = DeviceStreamHardwareConfigurationHardwareType(d.pop("hardwareType"))

        audio_hw_descriptor = d.pop("audioHwDescriptor", UNSET)

        rtsp_encoding_needed = d.pop("rtspEncodingNeeded", UNSET)

        is_onvif = d.pop("isOnvif", UNSET)

        ip_cam_username = d.pop("ipCamUsername", UNSET)

        ip_cam_password = d.pop("ipCamPassword", UNSET)

        _quality = d.pop("quality", UNSET)
        quality: Union[Unset, DeviceStreamHardwareConfigurationQuality]
        if isinstance(_quality, Unset):
            quality = UNSET
        else:
            quality = DeviceStreamHardwareConfigurationQuality(_quality)

        device_stream_hardware_configuration = cls(
            type=type,
            hw_descriptor=hw_descriptor,
            hardware_type=hardware_type,
            audio_hw_descriptor=audio_hw_descriptor,
            rtsp_encoding_needed=rtsp_encoding_needed,
            is_onvif=is_onvif,
            ip_cam_username=ip_cam_username,
            ip_cam_password=ip_cam_password,
            quality=quality,
        )

        device_stream_hardware_configuration.additional_properties = d
        return device_stream_hardware_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.location_view_configuration_basemap import LocationViewConfigurationBasemap
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.location_viewport import LocationViewport


T = TypeVar("T", bound="LocationViewConfiguration")


@attr.s(auto_attribs=True)
class LocationViewConfiguration:
    """
    Attributes:
        basemap (Union[Unset, LocationViewConfigurationBasemap]):
        viewport (Union[Unset, LocationViewport]):
    """

    basemap: Union[Unset, LocationViewConfigurationBasemap] = UNSET
    viewport: Union[Unset, "LocationViewport"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        basemap: Union[Unset, str] = UNSET
        if not isinstance(self.basemap, Unset):
            basemap = self.basemap.value

        viewport: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.viewport, Unset):
            viewport = self.viewport.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if basemap is not UNSET:
            field_dict["basemap"] = basemap
        if viewport is not UNSET:
            field_dict["viewport"] = viewport

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.location_viewport import LocationViewport

        d = src_dict.copy()
        _basemap = d.pop("basemap", UNSET)
        basemap: Union[Unset, LocationViewConfigurationBasemap]
        if isinstance(_basemap, Unset):
            basemap = UNSET
        else:
            basemap = LocationViewConfigurationBasemap(_basemap)

        _viewport = d.pop("viewport", UNSET)
        viewport: Union[Unset, LocationViewport]
        if isinstance(_viewport, Unset):
            viewport = UNSET
        else:
            viewport = LocationViewport.from_dict(_viewport)

        location_view_configuration = cls(
            basemap=basemap,
            viewport=viewport,
        )

        location_view_configuration.additional_properties = d
        return location_view_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import unittest
from formant.sdk.cloud.v2.src.admin_api import AdminAPI
from formant.sdk.cloud.v2.formant_admin_api_client.models import (
    DeviceQuery,
    PartialDevice,
    DeviceConfiguration,
    DeviceConfigurationDocument,
)
import os

EMAIL = os.getenv("FORMANT_EMAIL")
PASSWORD = os.getenv("FORMANT_PASSWORD")
BASE_URL = "https://api.formant.io/v1"


class TestDevices(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestDevices, self).__init__(*args, **kwargs)
        self.client = AdminAPI(email=EMAIL, password=PASSWORD, base_url=BASE_URL)

    def test_query_without_params(self):
        device_query = DeviceQuery()
        result = self.client.devices.query(device_query)
        self.assertEqual(result.status_code, 200)

    def test_query_with_params(self):
        json_params = {"name": "model00.001", "tags": {"location": ["sf", "la"]}}
        device_query = DeviceQuery(**json_params)
        result = self.client.devices.query(device_query)
        self.assertEqual(result.status_code, 200)

    def test_patch(self):
        device_id = "404e2d2c-f95f-41d8-bce1-915e314a6898"
        json_params = {"enabled": True}
        partial_device = PartialDevice(**json_params)
        result = self.client.devices.patch(device_id, partial_device)
        self.assertEqual(result.status_code, 200)

    def test_get_device(self):
        device_id = "404e2d2c-f95f-41d8-bce1-915e314a6898"
        result = self.client.devices.get_device(device_id)
        self.assertEqual(result.status_code, 200)

    def test_get_device_configuration(self):
        device_id = "404e2d2c-f95f-41d8-bce1-915e314a6898"
        desired_configuration_version = 3
        result = self.client.devices.get_device_configuration(
            device_id, desired_configuration_version
        )
        self.assertEqual(result.status_code, 200)

    def test_post_device_configuration(self):
        device_id = "404e2d2c-f95f-41d8-bce1-915e314a6898"
        device_configuration_document = DeviceConfigurationDocument()
        device_configuration = DeviceConfiguration(device_configuration_document)
        result = self.client.devices.post_device_configuration(
            device_id, device_configuration
        )
        self.assertEqual(result.status_code, 201)


unittest.main()



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.transform import Transform


T = TypeVar("T", bound="TransformNode")


@attr.s(auto_attribs=True)
class TransformNode:
    """
    Attributes:
        name (Union[Unset, str]):
        transform (Union[Unset, Transform]):
        children (Union[Unset, List['TransformNode']]):
        url (Union[Unset, str]):
        size (Union[Unset, int]):
    """

    name: Union[Unset, str] = UNSET
    transform: Union[Unset, "Transform"] = UNSET
    children: Union[Unset, List["TransformNode"]] = UNSET
    url: Union[Unset, str] = UNSET
    size: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        transform: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.transform, Unset):
            transform = self.transform.to_dict()

        children: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.children, Unset):
            children = []
            for children_item_data in self.children:
                children_item = children_item_data.to_dict()

                children.append(children_item)

        url = self.url
        size = self.size

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if transform is not UNSET:
            field_dict["transform"] = transform
        if children is not UNSET:
            field_dict["children"] = children
        if url is not UNSET:
            field_dict["url"] = url
        if size is not UNSET:
            field_dict["size"] = size

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.transform import Transform

        d = src_dict.copy()
        name = d.pop("name", UNSET)

        _transform = d.pop("transform", UNSET)
        transform: Union[Unset, Transform]
        if isinstance(_transform, Unset):
            transform = UNSET
        else:
            transform = Transform.from_dict(_transform)

        children = []
        _children = d.pop("children", UNSET)
        for children_item_data in _children or []:
            children_item = TransformNode.from_dict(children_item_data)

            children.append(children_item)

        url = d.pop("url", UNSET)

        size = d.pop("size", UNSET)

        transform_node = cls(
            name=name,
            transform=transform,
            children=children,
            url=url,
            size=size,
        )

        transform_node.additional_properties = d
        return transform_node

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_sheet_parse_result import GoogleSheetParseResult


T = TypeVar("T", bound="GoogleSpreadsheetInspection")


@attr.s(auto_attribs=True)
class GoogleSpreadsheetInspection:
    """
    Attributes:
        parse_url_failed (Union[Unset, bool]):
        spreadsheet_get_failed (Union[Unset, bool]):
        sheet_find_failed (Union[Unset, bool]):
        sheet_title (Union[Unset, str]):
        parse_result (Union[Unset, GoogleSheetParseResult]):
    """

    parse_url_failed: Union[Unset, bool] = UNSET
    spreadsheet_get_failed: Union[Unset, bool] = UNSET
    sheet_find_failed: Union[Unset, bool] = UNSET
    sheet_title: Union[Unset, str] = UNSET
    parse_result: Union[Unset, "GoogleSheetParseResult"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        parse_url_failed = self.parse_url_failed
        spreadsheet_get_failed = self.spreadsheet_get_failed
        sheet_find_failed = self.sheet_find_failed
        sheet_title = self.sheet_title
        parse_result: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parse_result, Unset):
            parse_result = self.parse_result.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if parse_url_failed is not UNSET:
            field_dict["parseUrlFailed"] = parse_url_failed
        if spreadsheet_get_failed is not UNSET:
            field_dict["spreadsheetGetFailed"] = spreadsheet_get_failed
        if sheet_find_failed is not UNSET:
            field_dict["sheetFindFailed"] = sheet_find_failed
        if sheet_title is not UNSET:
            field_dict["sheetTitle"] = sheet_title
        if parse_result is not UNSET:
            field_dict["parseResult"] = parse_result

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_sheet_parse_result import GoogleSheetParseResult

        d = src_dict.copy()
        parse_url_failed = d.pop("parseUrlFailed", UNSET)

        spreadsheet_get_failed = d.pop("spreadsheetGetFailed", UNSET)

        sheet_find_failed = d.pop("sheetFindFailed", UNSET)

        sheet_title = d.pop("sheetTitle", UNSET)

        _parse_result = d.pop("parseResult", UNSET)
        parse_result: Union[Unset, GoogleSheetParseResult]
        if isinstance(_parse_result, Unset):
            parse_result = UNSET
        else:
            parse_result = GoogleSheetParseResult.from_dict(_parse_result)

        google_spreadsheet_inspection = cls(
            parse_url_failed=parse_url_failed,
            spreadsheet_get_failed=spreadsheet_get_failed,
            sheet_find_failed=sheet_find_failed,
            sheet_title=sheet_title,
            parse_result=parse_result,
        )

        google_spreadsheet_inspection.additional_properties = d
        return google_spreadsheet_inspection

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CountHistoryQueryType(str, Enum):
    DATA_POINT = "data-point"
    BYTE = "byte"
    ASSET_BYTE = "asset-byte"
    ASSET = "asset"
    COST = "cost"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Authentication")


@attr.s(auto_attribs=True)
class Authentication:
    """
    Attributes:
        access_token (str):
        organization_id (str):
        user_id (str):
        refresh_token (Union[Unset, str]):
        is_superuser (Union[Unset, bool]):
    """

    access_token: str
    organization_id: str
    user_id: str
    refresh_token: Union[Unset, str] = UNSET
    is_superuser: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        access_token = self.access_token
        organization_id = self.organization_id
        user_id = self.user_id
        refresh_token = self.refresh_token
        is_superuser = self.is_superuser

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "accessToken": access_token,
                "organizationId": organization_id,
                "userId": user_id,
            }
        )
        if refresh_token is not UNSET:
            field_dict["refreshToken"] = refresh_token
        if is_superuser is not UNSET:
            field_dict["isSuperuser"] = is_superuser

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        access_token = d.pop("accessToken")

        organization_id = d.pop("organizationId")

        user_id = d.pop("userId")

        refresh_token = d.pop("refreshToken", UNSET)

        is_superuser = d.pop("isSuperuser", UNSET)

        authentication = cls(
            access_token=access_token,
            organization_id=organization_id,
            user_id=user_id,
            refresh_token=refresh_token,
            is_superuser=is_superuser,
        )

        authentication.additional_properties = d
        return authentication

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import unittest
from formant.sdk.cloud.v2.src.query_api import QueryAPI

import dateutil.parser as parser
import os

EMAIL = os.getenv("FORMANT_EMAIL")
PASSWORD = os.getenv("FORMANT_PASSWORD")
BASE_URL = "https://api.formant.io/v1"


class TestOnlineDevices(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestOnlineDevices, self).__init__(*args, **kwargs)
        self.client = QueryAPI(email=EMAIL, password=PASSWORD, base_url=BASE_URL)

    def test_online(self):
        response = self.client.online_devices.online()
        self.assertEqual(response.status_code, 200)


unittest.main()



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="UsagePrices")


@attr.s(auto_attribs=True)
class UsagePrices:
    """
    Attributes:
        data_points (float):
        assets (float):
        bytes_ (float):
        devices (float):
        rtc_turn (float):
        advanced_configuration (float):
        analytics (float):
        customer_portal (float):
        data_export (float):
        data_retention (float):
        support (float):
        teleop (float):
        observability (float):
        share (float):
        mission (float):
        diagnostics (float):
        ssh (float):
        spot (float):
    """

    data_points: float
    assets: float
    bytes_: float
    devices: float
    rtc_turn: float
    advanced_configuration: float
    analytics: float
    customer_portal: float
    data_export: float
    data_retention: float
    support: float
    teleop: float
    observability: float
    share: float
    mission: float
    diagnostics: float
    ssh: float
    spot: float
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        data_points = self.data_points
        assets = self.assets
        bytes_ = self.bytes_
        devices = self.devices
        rtc_turn = self.rtc_turn
        advanced_configuration = self.advanced_configuration
        analytics = self.analytics
        customer_portal = self.customer_portal
        data_export = self.data_export
        data_retention = self.data_retention
        support = self.support
        teleop = self.teleop
        observability = self.observability
        share = self.share
        mission = self.mission
        diagnostics = self.diagnostics
        ssh = self.ssh
        spot = self.spot

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "dataPoints": data_points,
                "assets": assets,
                "bytes": bytes_,
                "devices": devices,
                "rtcTurn": rtc_turn,
                "advancedConfiguration": advanced_configuration,
                "analytics": analytics,
                "customerPortal": customer_portal,
                "dataExport": data_export,
                "dataRetention": data_retention,
                "support": support,
                "teleop": teleop,
                "observability": observability,
                "share": share,
                "mission": mission,
                "diagnostics": diagnostics,
                "ssh": ssh,
                "spot": spot,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        data_points = d.pop("dataPoints")

        assets = d.pop("assets")

        bytes_ = d.pop("bytes")

        devices = d.pop("devices")

        rtc_turn = d.pop("rtcTurn")

        advanced_configuration = d.pop("advancedConfiguration")

        analytics = d.pop("analytics")

        customer_portal = d.pop("customerPortal")

        data_export = d.pop("dataExport")

        data_retention = d.pop("dataRetention")

        support = d.pop("support")

        teleop = d.pop("teleop")

        observability = d.pop("observability")

        share = d.pop("share")

        mission = d.pop("mission")

        diagnostics = d.pop("diagnostics")

        ssh = d.pop("ssh")

        spot = d.pop("spot")

        usage_prices = cls(
            data_points=data_points,
            assets=assets,
            bytes_=bytes_,
            devices=devices,
            rtc_turn=rtc_turn,
            advanced_configuration=advanced_configuration,
            analytics=analytics,
            customer_portal=customer_portal,
            data_export=data_export,
            data_retention=data_retention,
            support=support,
            teleop=teleop,
            observability=observability,
            share=share,
            mission=mission,
            diagnostics=diagnostics,
            ssh=ssh,
            spot=spot,
        )

        usage_prices.additional_properties = d
        return usage_prices

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

from ..models.metadata_type import MetadataType

if TYPE_CHECKING:
    from ..models.metadata_tags import MetadataTags


T = TypeVar("T", bound="Metadata")


@attr.s(auto_attribs=True)
class Metadata:
    """
    Attributes:
        device_id (str):
        name (str):
        type (MetadataType):
        tags (MetadataTags):
    """

    device_id: str
    name: str
    type: MetadataType
    tags: "MetadataTags"
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        device_id = self.device_id
        name = self.name
        type = self.type.value

        tags = self.tags.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "deviceId": device_id,
                "name": name,
                "type": type,
                "tags": tags,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.metadata_tags import MetadataTags

        d = src_dict.copy()
        device_id = d.pop("deviceId")

        name = d.pop("name")

        type = MetadataType(d.pop("type"))

        tags = MetadataTags.from_dict(d.pop("tags"))

        metadata = cls(
            device_id=device_id,
            name=name,
            type=type,
            tags=tags,
        )

        metadata.additional_properties = d
        return metadata

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.video_mime_type import VideoMimeType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Video")


@attr.s(auto_attribs=True)
class Video:
    """
    Attributes:
        url (str):
        duration (int):
        mime_type (VideoMimeType):
        size (Union[Unset, int]):
    """

    url: str
    duration: int
    mime_type: VideoMimeType
    size: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        url = self.url
        duration = self.duration
        mime_type = self.mime_type.value

        size = self.size

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "url": url,
                "duration": duration,
                "mimeType": mime_type,
            }
        )
        if size is not UNSET:
            field_dict["size"] = size

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        url = d.pop("url")

        duration = d.pop("duration")

        mime_type = VideoMimeType(d.pop("mimeType"))

        size = d.pop("size", UNSET)

        video = cls(
            url=url,
            duration=duration,
            mime_type=mime_type,
            size=size,
        )

        video.additional_properties = d
        return video

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import AdapterCascadingConfigurationListResponse, Adapter, AdapterCascadingConfigurationSpecificity, AdapterCascadingConfiguration, AdapterCascadingConfigurationType, AdapterListResponse, AdapterConfiguration
from formant.sdk.cloud.v2.formant_admin_api_client.api.adapter import adapter_controller_post, adapter_controller_delete
from formant.sdk.cloud.v2.formant_admin_api_client.types import *
from typing import Any



import unittest
from formant.sdk.cloud.v2.src.admin_api import AdminAPI
import os
import aiounittest
import uuid

EMAIL = os.getenv("FORMANT_EMAIL")
PASSWORD = os.getenv("FORMANT_PASSWORD")
BASE_URL = "https://api.formant.io/v1"


class TestFiles(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestFiles, self).__init__(*args, **kwargs)
        self.client = AdminAPI(email=EMAIL, password=PASSWORD, base_url=BASE_URL)

    def test_upload(self):
        random_name = str(uuid.uuid4())
        file = open(f"{random_name}.txt", "w")
        file.write("Sample Data")
        file.close()
        path = file.name
        result = self.client.files.upload(path)
        os.remove(path)
        self.assertTrue(result)


class TestFilesAsync(aiounittest.AsyncTestCase):
    def __init__(self, *args, **kwargs):
        super(TestFilesAsync, self).__init__(*args, **kwargs)
        self.client = AdminAPI(email=EMAIL, password=PASSWORD, base_url=BASE_URL)

    async def test_async_upload(self):
        random_name = str(uuid.uuid4())
        file = open(f"test_{random_name}.txt", "w")
        file.write("Sample Data")
        file.close()
        path = file.name
        result = await self.client.files.upload_async(path)
        os.remove(path)
        self.assertTrue(result)


unittest.main()



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

from ..models.on_demand_buffer_buffer_type import OnDemandBufferBufferType

if TYPE_CHECKING:
    from ..models.on_demand_stream_presence import OnDemandStreamPresence


T = TypeVar("T", bound="OnDemandBuffer")


@attr.s(auto_attribs=True)
class OnDemandBuffer:
    """
    Attributes:
        buffer_type (OnDemandBufferBufferType):
        streams (List['OnDemandStreamPresence']):
    """

    buffer_type: OnDemandBufferBufferType
    streams: List["OnDemandStreamPresence"]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        buffer_type = self.buffer_type.value

        streams = []
        for streams_item_data in self.streams:
            streams_item = streams_item_data.to_dict()

            streams.append(streams_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "bufferType": buffer_type,
                "streams": streams,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.on_demand_stream_presence import OnDemandStreamPresence

        d = src_dict.copy()
        buffer_type = OnDemandBufferBufferType(d.pop("bufferType"))

        streams = []
        _streams = d.pop("streams")
        for streams_item_data in _streams:
            streams_item = OnDemandStreamPresence.from_dict(streams_item_data)

            streams.append(streams_item)

        on_demand_buffer = cls(
            buffer_type=buffer_type,
            streams=streams,
        )

        on_demand_buffer.additional_properties = d
        return on_demand_buffer

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
